@@ -1,13 +1,11 @@
 # -*- coding: utf-8 -*-
 
 import os
-import shutil
-from tempfile import mkstemp
 
 from mutagen._compat import BytesIO
-from mutagen.oggopus import OggOpus, OggOpusInfo, delete
+from mutagen.oggopus import OggOpus, OggOpusInfo, delete, error
 from mutagen.ogg import OggPage
-from tests import TestCase, DATA_DIR
+from tests import TestCase, DATA_DIR, get_temp_copy
 from tests.test_ogg import TOggFileTypeMixin
 
 
@@ -15,10 +13,7 @@ class TOggOpus(TestCase, TOggFileTypeMixin):
     Kind = OggOpus
 
     def setUp(self):
-        original = os.path.join(DATA_DIR, "example.opus")
-        fd, self.filename = mkstemp(suffix='.opus')
-        os.close(fd)
-        shutil.copy(original, self.filename)
+        self.filename = get_temp_copy(os.path.join(DATA_DIR, "example.opus"))
         self.audio = self.Kind(self.filename)
 
     def tearDown(self):
@@ -41,12 +36,14 @@ class TOggOpus(TestCase, TOggFileTypeMixin):
         self.failUnless("audio/ogg; codecs=opus" in self.audio.mime)
 
     def test_invalid_not_first(self):
-        page = OggPage(open(self.filename, "rb"))
+        with open(self.filename, "rb") as h:
+            page = OggPage(h)
         page.first = False
-        self.failUnlessRaises(IOError, OggOpusInfo, BytesIO(page.write()))
+        self.failUnlessRaises(error, OggOpusInfo, BytesIO(page.write()))
 
     def test_unsupported_version(self):
-        page = OggPage(open(self.filename, "rb"))
+        with open(self.filename, "rb") as h:
+            page = OggPage(h)
         data = bytearray(page.packets[0])
 
         data[8] = 0x03
@@ -55,7 +52,7 @@ class TOggOpus(TestCase, TOggFileTypeMixin):
 
         data[8] = 0x10
         page.packets[0] = bytes(data)
-        self.failUnlessRaises(IOError, OggOpusInfo, BytesIO(page.write()))
+        self.failUnlessRaises(error, OggOpusInfo, BytesIO(page.write()))
 
     def test_preserve_non_padding(self):
         self.audio["FOO"] = ["BAR"]
