@@ -5,7 +5,7 @@ import glob
 
 from tests.test_tools import _TTools
 
-from mutagen._toolsutil import fsnative as fsn
+from mutagen._senf import fsnative as fsn
 
 
 class TMutagenInspect(_TTools):
